
from odoo import api
from odoo.tests import tagged

from odoo.addons.loempia.tests.common import MockRequest, TestLoempia
from odoo.addons.loempia.controllers.frontend import AppsFrontendController

@tagged('post_install', '-at_install')
class TestLoempiaRating(TestLoempia):

    @classmethod
    def setUpClass(self):
        super(TestLoempiaRating, self).setUpClass()
        self.Controller = AppsFrontendController()

    def _create_rating(self, module_id):
        return self.env['loempia.module.rating'].create({
            'module_id': module_id.id,
            'template_id': module_id.template_id.id,
            'rating': 3,
            'subject': 'test',
            'body': 'test',
            'author_id': self.customer.partner_id.id,
        })

    def test_01_rating(self):
        """
        This test is composed of four separated tests.
        The first test ensure that you can't a negative directly.
        The second test ensure that you can still post a negative rating aferwards.
        The third test ensure that you can post a comment on another version of the same product.
        The last test consist to ensure that you can post multiple comments but only one rating on a module you bought.
        """

        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)) as r:
            # 01
            self.assertEqual(r.env.user.id, self.customer.id, msg='Ensure correct user')
            self.Controller.post_rating(self.module_12_A.id, rating='2', subject="test_01_rating_subject", comment="test_01_rating_body")
            self.assertEqual(
                len(self.module_12_A.ratings_ids),
                0,
                msg="A rating below 3 has to contains 'force_post' to be posted"
            )

            # 02
            self.assertEqual(r.env.user.id, self.customer.id, msg='Ensure correct user')
            self.Controller.post_rating(
                self.module_12_A.id,
                rating='2',
                subject="test_01_rating_subject",
                comment="test_01_rating_body",
                force_post=True
            )
            self.assertEqual(
                len(self.module_12_A.ratings_ids),
                1,
                msg="A rating must be posted"
            )
            rating_id = self.module_12_A.ratings_ids[0]
            self.assertEqual(rating_id.rating, 2, msg='Rating must be the same as the one posted')
            self.assertEqual(rating_id.subject, "test_01_rating_subject", msg='Subject needs to be correctly posted')
            self.assertTrue("test_01_rating_body" in rating_id.body, msg='Body needs to be correctly posted')  # html tags
            self.assertEqual(
                rating_id.author_id.id,
                self.customer.partner_id.id,
                msg='Author must be correct'
            )

            # 03
            self.assertEqual(r.env.user.id, self.customer.id, msg='Ensure correct user')
            self.Controller.post_rating(
                self.module_11_A.id,
                rating='4',
                subject="test_01_rating_subject",
                comment="<a href='http://odoo.com'>test_01_rating_body</a>",
            )
            self.assertEqual(
                len(self.module_11_A.ratings_ids),
                1,
                msg="A rating must posted"
            )
            rating_id = self.module_11_A.ratings_ids[0]
            self.assertEqual(rating_id.rating, 4, msg='Rating must be the same as the one posted')
            self.assertEqual(self.template_A.average_rating, 0, msg='Average rating must be equal to 0 because we have less than 5 ratings')
            sanitized_body = '<span href="">test_01_rating_body</span>'
            self.assertEqual(rating_id.body, sanitized_body)  # Test for sanitize comment

            # 04
            self.assertEqual(r.env.user.id, self.customer.id, msg='Ensure correct user')
            self.Controller.post_rating(
                self.module_11_A.id,
                rating='10',
                subject="test_01_rating_subject_second",
                comment="test_01_rating_body_second",
            )
            self.assertEqual(
                len(self.module_11_A.ratings_ids),
                2,
                msg="A rating already exists, there should be a comment and a rating now."
            )
            self.assertEqual(
                len(self.module_11_A.ratings_ids.filtered(lambda r: r.rating != 0)),
                1,
                msg="A rating already exists, there should be a comment and a rating now."
            )
            rating_id = self.module_11_A.ratings_ids.filtered(lambda r: r.rating != 0)[0]  # Same user so only one rating stays
            self.assertEqual(rating_id.rating, 5, msg='Maximal rating is 5')

    def test_02_rating(self):
        """
        The test ensures that you can only post comment if you didn't have bought the module and
        that you can answer to a comment if you are the maintainer of the module.
        """

        env = api.Environment(self.env.cr, self.maintainer.id, {})
        with MockRequest(env, website=self.website.with_env(env)) as r:
            # 01
            self.assertEqual(r.env.user.id, self.maintainer.id, msg='Ensure correct user')
            self.Controller.post_rating(self.module_11_A.id, rating='4', subject="test_02_rating_subject", comment="test_02_rating_body")
            self.assertEqual(
                self.module_11_A.ratings_ids[0].rating,
                0,
                msg="The maintainer does not purchase the module, so the rating should be set to 0"
            )

            # 02
            rating = self._create_rating(self.module_12_A)
            self.assertEqual(r.env.user.id, self.maintainer.id, msg='Ensure correct user')
            self.Controller.post_reply(parent_id=rating.id, subject="test_02_rating_subject", comment="test_02_rating_body")
            self.assertEqual(len(self.module_12_A.ratings_ids), 2, msg='One rating, one reply')
            reply = self.module_12_A.ratings_ids.filtered(lambda r: r.parent_id.id == rating.id)[0]
            self.assertEqual(reply.rating, 0, msg="A reply should not have any rating.")

    def test_average_rating(self):
        """
        This test has for goal to ensure that after 5 ratings, the average rating is
        correclty computed.
        """
        env = api.Environment(self.env.cr, self.customer.id, {})
        with MockRequest(env, website=self.website.with_env(env)) as r:
            # 01
            self.assertEqual(r.env.user.id, self.customer.id, msg='Ensure correct user')
            self.Controller.post_rating(
                self.module_12_B.id,
                rating='4',
                subject="test_03_average_rating_subject",
                comment="test_03_average_rating_body"
            )
            self.assertEqual(
                len(self.template_B.module_ids.mapped('ratings_ids')),
                5,
                msg="We ensure that there is 5 rating, so average rating can be computed"
            )
            self.assertEqual(
                self.template_B.average_rating,
                4,
                msg="All the rating are equal to 4, so we should have 4 as average rating"
            )

    def test_report_rating(self):
        """
        This test ensure that the report of rating works as well as the report count.
        """
        rating = self._create_rating(self.module_12_A)
        env = api.Environment(self.env.cr, self.functional.id, {})
        with MockRequest(env, website=self.website.with_env(env)) as r:
            # 01
            self.assertEqual(r.env.user.id, self.functional.id, msg='Ensure correct user')
            self.Controller.report_rating(rating_id=rating.id, reason="test_04_report_rating")
            self.assertEqual(rating.state, 'pending', msg="The report has to be reviewed")
            self.assertEqual(rating.report_count, 1, msg="THe count of number should be 1")

            # 02
            self.assertEqual(r.env.user.id, self.functional.id, msg='Ensure correct user')
            self.Controller.report_rating(rating_id=rating.id, reason="test_04_report_rating")
            self.assertEqual(rating.state, 'pending', msg="The report still has to be reviewed.")
            self.assertEqual(rating.report_count, 2, msg="The report count should increase by 1.")
